/*
 * fragmentation_helpers.hpp
 *
 *  Created on: Oct 18, 2011
 *      Author: heber
 */

#ifndef FRAGMENTATION_HELPERS_HPP_
#define FRAGMENTATION_HELPERS_HPP_


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <list>
#include <map>
#include <string>
#include <vector>

#include "graph.hpp"

class bond;
class molecule;

bool ScanBufferIntoKeySet(char *buffer, KeySet &CurrentSet);
bool ParseKeySetFile(std::string &path, Graph *&FragmentList);
bool ParseTEFactorsFile(char *path, Graph *FragmentList);
bool StoreKeySetFile(Graph &KeySetList, std::string &path);
bool StoreTEFactorsFile(Graph &KeySetList, char *path);
std::map<int,KeySet> * GraphToIndexedKeySet(Graph *GlobalKeySetList);
void InsertIntoAdaptiveCriteriaList(std::map<int, std::pair<double,int> > *AdaptiveCriteriaList, std::map<int,KeySet> &IndexKeySetList, int FragOrder, int No, double Value);
int CountLinesinFile(std::ifstream &InputFile);
std::map<int, std::pair<double,int> > * ScanAdaptiveFileIntoMap(std::string &path, std::map<int,KeySet> &IndexKeySetList);
std::map<double, std::pair<int,int> >  * ReMapAdaptiveCriteriaListToValue(std::map<int, std::pair<double,int> > *AdaptiveCriteriaList, molecule *mol);
bool MarkUpdateCandidates(bool *AtomMask, std::map<double, std::pair<int,int> > &FinalRootCandidates, int Order, molecule *mol);
void PrintAtomMask(bool *AtomMask, int AtomCount);
void SPFragmentGenerator_ClearingTouched(int verbosity, int *&TouchedList, int SubOrder, int &TouchedIndex);
int AddPowersetToSnakeStack(int verbosity, int CurrentCombination, int SetDimension, KeySet *FragmentSet, std::vector<bond *> &BondsSet, int *&TouchedList, int &TouchedIndex);
int CountSetMembers(std::list<bond *>::const_iterator SetFirst, std::list<bond *>::const_iterator SetLast, int *&TouchedList, int TouchedIndex);
int FillBondsList(std::vector<bond *> &BondsList, std::list<bond *>::const_iterator SetFirst, std::list<bond *>::const_iterator SetLast, int *&TouchedList, int TouchedIndex);
void RemoveAllTouchedFromSnakeStack(int verbosity, KeySet *FragmentSet, int *&TouchedList, int &TouchedIndex);
void InitialiseSPList(int Order, struct UniqueFragments &FragmentSearch);
void FreeSPList(int Order, struct UniqueFragments &FragmentSearch);
void SetSPList(int Order, struct UniqueFragments &FragmentSearch);
void ResetSPList(int Order, struct UniqueFragments &FragmentSearch);
void FillSPListandLabelVertices(int Order, struct UniqueFragments &FragmentSearch, molecule *mol, KeySet RestrictedKeySet);
void OutputSPList(int Order, struct UniqueFragments &FragmentSearch);
int CountNumbersInBondsList(int Order, struct UniqueFragments &FragmentSearch);
int CombineAllOrderListIntoOne(Graph *&FragmentList, Graph ***FragmentLowerOrdersList, KeyStack &RootStack, molecule *mol);
void FreeAllOrdersList(Graph ***FragmentLowerOrdersList, KeyStack &RootStack, molecule *mol);

#endif /* FRAGMENTATION_HELPERS_HPP_ */
